@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,13 +13,11 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 from collections import defaultdict
 
 import attr
 
-from hypothesis.internal.compat import hbytes, hrange, int_from_bytes, int_to_bytes
+from hypothesis.internal.compat import int_from_bytes, int_to_bytes
 from hypothesis.internal.conjecture.choicetree import ChoiceTree
 from hypothesis.internal.conjecture.data import ConjectureResult, Overrun, Status
 from hypothesis.internal.conjecture.floats import (
@@ -67,7 +63,7 @@ SHRINK_PASS_DEFINITIONS = {}  # type: Dict[str, ShrinkPassDefinition]
 
 
 @attr.s()
-class ShrinkPassDefinition(object):
+class ShrinkPassDefinition:
     """A shrink pass bundles together a large number of local changes to
     the current shrink target.
 
@@ -100,7 +96,7 @@ def defines_shrink_pass():
         ShrinkPassDefinition(run_with_chooser=run_step)
 
         def run(self):  # pragma: no cover
-            assert False, "Shrink passes should not be run directly"
+            raise AssertionError("Shrink passes should not be run directly")
 
         run.__name__ = run_step.__name__
         run.is_shrink_pass = True
@@ -109,7 +105,7 @@ def defines_shrink_pass():
     return accept
 
 
-class Shrinker(object):
+class Shrinker:
     """A shrinker is a child object of a ConjectureRunner which is designed to
     manage the associated state of a particular shrink problem. That is, we
     have some initial ConjectureData object and some property of interest
@@ -271,7 +267,7 @@ class Shrinker(object):
         Note that initial is a ConjectureData object, and predicate
         takes ConjectureData objects.
         """
-        self.__engine = engine
+        self.engine = engine
         self.__predicate = predicate
         self.__derived_values = {}
         self.__pending_shrink_explanation = None
@@ -284,7 +280,7 @@ class Shrinker(object):
         self.update_shrink_target(initial)
         self.shrinks = 0
 
-        self.initial_calls = self.__engine.call_count
+        self.initial_calls = self.engine.call_count
 
         self.passes_by_name = {}
         self.passes = []
@@ -335,12 +331,12 @@ class Shrinker(object):
     def calls(self):
         """Return the number of calls that have been made to the underlying
         test function."""
-        return self.__engine.call_count
+        return self.engine.call_count
 
     def consider_new_buffer(self, buffer):
         """Returns True if after running this buffer the result would be
         the current shrink_target."""
-        buffer = hbytes(buffer)
+        buffer = bytes(buffer)
         return buffer.startswith(self.buffer) or self.incorporate_new_buffer(buffer)
 
     def incorporate_new_buffer(self, buffer):
@@ -348,7 +344,7 @@ class Shrinker(object):
         that changed the shrink_target, or determines that doing so would
         be useless and returns False without running it."""
 
-        buffer = hbytes(buffer[: self.shrink_target.index])
+        buffer = bytes(buffer[: self.shrink_target.index])
         # Sometimes an attempt at lexicographic minimization will do the wrong
         # thing because the buffer has changed under it (e.g. something has
         # turned into a write, the bit size has changed). The result would be
@@ -388,17 +384,17 @@ class Shrinker(object):
             self.debug(self.__pending_shrink_explanation)
             self.__pending_shrink_explanation = None
 
-        buffer = hbytes(buffer)
-        result = self.__engine.cached_test_function(buffer)
+        buffer = bytes(buffer)
+        result = self.engine.cached_test_function(buffer)
         self.incorporate_test_data(result)
         return result
 
     def debug(self, msg):
-        self.__engine.debug(msg)
+        self.engine.debug(msg)
 
     @property
     def random(self):
-        return self.__engine.random
+        return self.engine.random
 
     def shrink(self):
         """Run the full set of shrinks and update shrink_target.
@@ -415,14 +411,14 @@ class Shrinker(object):
         # operating on a block of all zero bytes so can use non-zeroness as a
         # signpost of complexity).
         if not any(self.shrink_target.buffer) or self.incorporate_new_buffer(
-            hbytes(len(self.shrink_target.buffer))
+            bytes(len(self.shrink_target.buffer))
         ):
             return
 
         try:
             self.greedy_shrink()
         finally:
-            if self.__engine.report_debug_info:
+            if self.engine.report_debug_info:
 
                 def s(n):
                     return "s" if n != 1 else ""
@@ -433,7 +429,7 @@ class Shrinker(object):
                 self.debug("Shrink pass profiling")
                 self.debug("---------------------")
                 self.debug("")
-                calls = self.__engine.call_count - self.initial_calls
+                calls = self.engine.call_count - self.initial_calls
                 self.debug(
                     (
                         "Shrinking made a total of %d call%s "
@@ -503,6 +499,7 @@ class Shrinker(object):
                 "minimize_duplicated_blocks",
                 block_program("-XX"),
                 "minimize_individual_blocks",
+                block_program("--X"),
             ]
         )
 
@@ -631,10 +628,13 @@ class Shrinker(object):
 
         ls = self.examples_by_label[label]
 
-        i = chooser.choose(hrange(len(ls) - 1))
+        i = chooser.choose(range(len(ls) - 1))
 
         ancestor = ls[i]
 
+        if i + 1 == len(ls) or ls[i + 1].start >= ancestor.end:
+            return
+
         @self.cached(label, i)
         def descendants():
             lo = i + 1
@@ -645,10 +645,14 @@ class Shrinker(object):
                     hi = mid
                 else:
                     lo = mid
-            return ls[i + 1 : hi]
+            return [t for t in ls[i + 1 : hi] if t.length < ancestor.length]
 
         descendant = chooser.choose(descendants, lambda ex: ex.length > 0)
 
+        assert ancestor.start <= descendant.start
+        assert ancestor.end >= descendant.end
+        assert descendant.length < ancestor.length
+
         self.incorporate_new_buffer(
             self.buffer[: ancestor.start]
             + self.buffer[descendant.start : descendant.end]
@@ -708,14 +712,14 @@ class Shrinker(object):
         offset = min(ints)
         assert offset > 0
 
-        for i in hrange(len(ints)):
+        for i in range(len(ints)):
             ints[i] -= offset
 
         def reoffset(o):
             new_blocks = list(blocked)
             for i, v in zip(changed, ints):
                 new_blocks[i] = int_to_bytes(v + o, len(blocked[i]))
-            return self.incorporate_new_buffer(hbytes().join(new_blocks))
+            return self.incorporate_new_buffer(b"".join(new_blocks))
 
         Integer.shrink(offset, reoffset, random=self.random)
         self.clear_change_tracking()
@@ -766,7 +770,7 @@ class Shrinker(object):
 
                 # Between these two changed regions we now do a linear scan to
                 # check if any specific block values have changed.
-                for i in hrange(first_changed, last_changed + 1):
+                for i in range(first_changed, last_changed + 1):
                     u, v = blocks.bounds(i)
                     if i not in self.__all_changed_blocks and prev[u:v] != new[u:v]:
                         self.__all_changed_blocks.add(i)
@@ -815,9 +819,9 @@ class Shrinker(object):
 
         initial_data = self.cached_test_function(initial_attempt)
 
-        if initial_data.status == Status.INTERESTING:
+        if initial_data is self.shrink_target:
             self.lower_common_block_offset()
-            return initial_data is self.shrink_target
+            return True
 
         # If this produced something completely invalid we ditch it
         # here rather than trying to persevere.
@@ -974,15 +978,13 @@ class Shrinker(object):
             )
 
         to_right = find_integer(lambda n: delete_region(j, j + n))
-
-        if to_right > 0:
-            find_integer(lambda n: delete_region(j - n, j + to_right))
+        find_integer(lambda n: delete_region(j - n, j + to_right))
 
     def try_zero_example(self, ex):
         u = ex.start
         v = ex.end
         attempt = self.cached_test_function(
-            self.buffer[:u] + hbytes(v - u) + self.buffer[v:]
+            self.buffer[:u] + bytes(v - u) + self.buffer[v:]
         )
 
         if attempt is Overrun:
@@ -994,7 +996,7 @@ class Shrinker(object):
         if attempt is not self.shrink_target:
             if in_replacement.end < len(attempt.buffer) and used < ex.length:
                 self.incorporate_new_buffer(
-                    self.buffer[:u] + hbytes(used) + self.buffer[v:]
+                    self.buffer[:u] + bytes(used) + self.buffer[v:]
                 )
         return self.examples[ex.index].trivial
 
@@ -1215,7 +1217,7 @@ class Shrinker(object):
 
         ex = self.examples[
             chooser.choose(
-                hrange(first_example_after_block, len(self.examples)),
+                range(first_example_after_block, len(self.examples)),
                 lambda i: self.examples[i].length > 0,
             )
         ]
@@ -1285,7 +1287,7 @@ class Shrinker(object):
             if d < 0:
                 return False
 
-            if self.consider_new_buffer(hbytes([d if b == c else b for b in buf])):
+            if self.consider_new_buffer(bytes([d if b == c else b for b in buf])):
                 if d <= 1:
                     # For small values of d if this succeeds we take this
                     # as evidence that it is worth doing a a bulk replacement
@@ -1304,7 +1306,7 @@ class Shrinker(object):
                             return c - k <= b <= c and d < b
 
                         return self.consider_new_buffer(
-                            hbytes([d if should_replace_byte(b) else b for b in buf])
+                            bytes([d if should_replace_byte(b) else b for b in buf])
                         )
 
                     find_integer(replace_range)
@@ -1361,7 +1363,7 @@ class Shrinker(object):
         if i + len(description) > len(original.blocks) or i < 0:
             return False
         attempt = bytearray(original.buffer)
-        for _ in hrange(repeats):
+        for _ in range(repeats):
             for k, d in reversed(list(enumerate(description))):
                 j = i + k
                 u, v = original.blocks[j].bounds
@@ -1376,7 +1378,7 @@ class Shrinker(object):
                 elif d == "X":
                     del attempt[u:v]
                 else:  # pragma: no cover
-                    assert False, "Unrecognised command %r" % (d,)
+                    raise AssertionError("Unrecognised command %r" % (d,))
         return self.incorporate_new_buffer(attempt)
 
 
@@ -1405,7 +1407,7 @@ def block_program(description):
             """Adaptively attempt to run the block program at the current
             index. If this successfully applies the block program ``k`` times
             then this runs in ``O(log(k))`` test function calls."""
-            i = chooser.choose(hrange(len(self.shrink_target.blocks) - n))
+            i = chooser.choose(range(len(self.shrink_target.blocks) - n))
             # First, run the block program at the chosen index. If this fails,
             # don't do any extra work, so that failure is as cheap as possible.
             if not self.run_block_program(i, description, original=self.shrink_target):
@@ -1443,7 +1445,7 @@ def block_program(description):
 
 
 @attr.s(slots=True, eq=False)
-class ShrinkPass(object):
+class ShrinkPass:
     run_with_chooser = attr.ib()
     index = attr.ib()
     shrinker = attr.ib()
