@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,7 +13,5 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
-__version_info__ = (4, 41, 3)
+__version_info__ = (5, 10, 0)
 __version__ = ".".join(map(str, __version_info__))
